#include "ChipSize.h"

ChipSize& ChipSize::operator = ( const ChipSize& rhs ) {

  // check for assignment against itself
  if ( this == &rhs )
    return *this;

  name = rhs.name;
  id = rhs.id;

  for ( size_t ii = 0; ii < 2; ii++ ) {

    chip_size_pix[ ii ] = rhs.chip_size_pix[ ii ];
    pixel_size_micron[ ii ] = rhs.pixel_size_micron[ ii ];
    chip_size_mm[ ii ] = rhs.chip_size_mm[ ii ];

  }

}

ChipSize::ChipSize( const char* header[], RDBColumn** rdb_column )
  throw( Exception ) {

  try {

    init_ChipSize( header, rdb_column );

  } catch( Exception& e ) {

    throw;

  }

}

void ChipSize::init_ChipSize( const char* header[],
			      RDBColumn** rdb_column )
  throw( Exception ) {

  try {

    for( size_t num = 0; NULL != header[ num ]; num++ ) {

      if ( 0 == strcmp( header[ num ], "chip" ) )
	simpleRDBTable< ChipSize >::getData( rdb_column[ num ], name );

      else if ( 0 == strcmp( header[ num ], "id" ) )
	simpleRDBTable< ChipSize >::getData( rdb_column[ num ], id );

      else if ( 0 == strcmp( header[ num ], "xsize_pix" ) )
	simpleRDBTable< ChipSize >::getData( rdb_column[ num ],
					     chip_size_pix[ XXX ] );
      else if ( 0 == strcmp( header[ num ], "ysize_pix" ) )
	simpleRDBTable< ChipSize >::getData( rdb_column[ num ],
					     chip_size_pix[ YYY ] );
      else if ( 0 == strcmp( header[ num ], "xpixelsize" ) )
	simpleRDBTable< ChipSize >::getData( rdb_column[ num ],
					     pixel_size_micron[ XXX ] );
      else if ( 0 == strcmp( header[ num ], "ypixelsize" ) )
	simpleRDBTable< ChipSize >::getData( rdb_column[ num ],
					     pixel_size_micron[ YYY ] );
      else if ( 0 == strcmp( header[ num ], "xsize_mm" ) )
	simpleRDBTable< ChipSize >::getData( rdb_column[ num ],
					     chip_size_mm[ XXX ] );
      else if ( 0 == strcmp( header[ num ], "ysize_mm" ) )
	simpleRDBTable< ChipSize >::getData( rdb_column[ num ],
					     chip_size_mm[ YYY ] );
      else {

	throw Exception( "ChipSize::init_ChipSize( const char**, RDBColumn**) "
			 ": Unrecognized option\n" );

      }

    }

  } catch( Exception& e ) {

    throw;

  }

}

void ChipSize::print( ostream& os ) const {

  os << name << '\t';
  os << id << '\t';
  os << chip_size_pix[ 0 ] << '\t';
  os << chip_size_pix[ 1 ] << '\t';
  os << pixel_size_micron[ 0 ] << '\t';
  os << pixel_size_micron[ 1 ] << '\t';
  os << chip_size_mm[ 0 ] << '\t';
  os << chip_size_mm[ 1 ];

}

