#ifndef simpleRDBTable_H
#define simpleRDBTable_H

#include <iostream>

using namespace std;

#include <suplibxx/str.h>
#include <Exception/Exception.h>

#include <rdbxx/RDBColumn.h>
#include <rdbxx/RDB.h>

/**
 *
 * A simple interface to the RDB++ read a row of an rdb table to initialize
 * a class Type. The class Type must have a constructor with the following
 * prototype:
 *
 *  Type( const char* header[], RDBColumn** rdb_column ) throw( Exception )
 *
 */
template< class Type >
class simpleRDBTable : public RDB {

  friend ostream& operator << ( ostream& os, simpleRDBTable< Type >& a ) {
    a.print( os );
    return os;
  }

  friend ostream& operator << ( ostream& os, simpleRDBTable< Type >* a ) {
    a->print( os );
    return os;
  }

public:

  ~simpleRDBTable( );

  simpleRDBTable( const string& fname, const int mode,
		  const char* header[] )
    throw( Exception );

  /// Given an RDBColumn, get a string value
  static void getData( RDBColumn* rdbcol, string& val ) throw ( Exception );

  /// Given an RDBColumn, get an integer value
  static void getData( RDBColumn* rdbcol, int& val ) throw ( Exception );

  /// Given an RDBColumn, get a long value
  static void getData( RDBColumn* rdbcol, long& val ) throw ( Exception );

  /// Given an RDBColumn, get a double value
  static void getData( RDBColumn* rdbcol, double& val ) throw ( Exception );

  void print( ostream& os ) const;

  /// Read a row of the RDB table.
  Type* readRow( ) throw( Exception );

protected:

  const char** rdb_header;

  RDBColumn** rdb_column;

private:

  /// Following the lead of the base class.
  simpleRDBTable( ) : rdb_header( NULL ), rdb_column( NULL ) { }

  // Declare but (purposedly) not define
  simpleRDBTable< Type >( const simpleRDBTable< Type >& c );

  // Declare but (purposedly) not define
  simpleRDBTable< Type >& operator = ( const simpleRDBTable< Type >& rhs );

  size_t count_num_args( const char* str[] );

};

#ifdef TEMPLATE_INCLUDE
#include "simpleRDBTable.cc"
#endif

#endif
