#include <cstdlib>
#include <cerrno>
#include <iostream>
#include <testlib/testlib.h>
#include <rdbxx/RDBColumnTmplt.h>
#include <rdbxx/RDBErr.h>

using namespace std;

int main( int argc, char* argv[] ) {
  START( "RDBColumnTmplt Test Suite" );

  RDBDoubleColumn dblcol( "DOUBLE", "N" );
  RDBLongColumn lngcol( "LONG", "N" );
  RDBStringColumn strcol( "STRING", "S" );

  RDBColumn* cols[3];
  cols[0] = &dblcol;
  cols[1] = &lngcol;
  cols[2] = &strcol;

  cols[0]->setThrow( false );
  cols[1]->setThrow( false );
  cols[2]->setThrow( false );

  double d = 3.141;
  long l = 9876;
  string s = "141.3";

  try {
    cols[0]->setData( d );
    TEST( "RDBColumnTmplt::getDataDouble dbl->dblcol->dbl", dblcol.getDataDouble( ), d );
    TEST( "RDBColumnTmplt::getDataLong   dbl->dblcol->lng", dblcol.getDataLong( ), 3 );
    string output = dblcol.getDataString( );
    string expected( "3.141" );
    TEST( "RDBColumnTmplt::getDataString dbl->dblcol->str", output, expected );

    cols[0]->setData( l );
    TEST( "RDBColumnTmplt::getDataDouble lng->dblcol->dbl", dblcol.getDataDouble( ), 9876.0 );
    TEST( "RDBColumnTmplt::getDataLong   lng->dblcol->lng", dblcol.getDataLong( ), l );
    TEST( "RDBColumnTmplt::getDataString lng->dblcol->str", dblcol.getDataString( ), "9876" );
    cols[0]->setData( s );
    TEST( "RDBColumnTmplt::getDataDouble str->dblcol->dbl", dblcol.getDataDouble( ), 141.3 );
    TEST( "RDBColumnTmplt::getDataLong   str->dblcol->lng", dblcol.getDataLong( ), 141 );
    TEST( "RDBColumnTmplt::getDataString str->dblcol->str", dblcol.getDataString( ), s );
    
    cols[1]->setData( d );
    TEST( "RDBColumnTmplt::getDataDouble dbl->lngcol->dbl", lngcol.getDataDouble( ), 3.0 );
    TEST( "RDBColumnTmplt::getDataLong   dbl->lngcol->lng", lngcol.getDataLong( ), 3 );
    TEST( "RDBColumnTmplt::getDataString dbl->lngcol->str", lngcol.getDataString( ), "3" );
    cols[1]->setData( l );
    TEST( "RDBColumnTmplt::getDataDouble lng->lngcol->dbl", lngcol.getDataDouble( ), 9876.0 );
    TEST( "RDBColumnTmplt::getDataLong   lng->lngcol->lng", lngcol.getDataLong( ), l );
    TEST( "RDBColumnTmplt::getDataString lng->lngcol->str", lngcol.getDataString( ), "9876" );
    cols[1]->setData( s );
    TEST( "RDBColumnTmplt::getDataDouble str->lngcol->dbl", lngcol.getDataDouble( ), 141.0 );
    TEST( "RDBColumnTmplt::getDataLong   str->lngcol->lng", lngcol.getDataLong( ), 141 );
    TEST( "RDBColumnTmplt::getDataString str->lngcol->str", lngcol.getDataString( ), "141" );
    
    cols[2]->setData( d );
    TEST( "RDBColumnTmplt::getDataDouble dbl->strcol->dbl", strcol.getDataDouble( ), d );
    TEST( "RDBColumnTmplt::getDataLong   dbl->strcol->lng", strcol.getDataLong( ), 3 );
    TEST( "RDBColumnTmplt::getDataString dbl->strcol->str", strcol.getDataString( ), "3.141" );
    cols[2]->setData( l );
    TEST( "RDBColumnTmplt::getDataDouble lng->strcol->dbl", strcol.getDataDouble( ), 9876.0 );
    TEST( "RDBColumnTmplt::getDataLong   lng->strcol->lng", strcol.getDataLong( ), l );
    TEST( "RDBColumnTmplt::getDataString lng->strcol->str", strcol.getDataString( ), "9876" );
    cols[2]->setData( s );
    TEST( "RDBColumnTmplt::getDataDouble str->strcol->dbl", strcol.getDataDouble( ), 141.3 );
    TEST( "RDBColumnTmplt::getDataLong   str->strcol->lng", strcol.getDataLong( ), 141 );
    TEST( "RDBColumnTmplt::getDataString str->strcol->str", strcol.getDataString( ), s );
    
  } catch ( RDBErr& re ) {
    cerr << "Caught an RDBErr `" << re << "'" << endl;

  } catch ( ... ) {
    cerr << "Caught something else..." << endl;

  }

  SUMMARY( );
  
  return FAILED( ) ? EXIT_FAILURE : EXIT_SUCCESS;
}
